@@ -19,10 +19,6 @@
 #ifndef LLVM_ANALYSIS_CONSTANTFOLDING_H
 #define LLVM_ANALYSIS_CONSTANTFOLDING_H
 
-#ifdef __sun
-#undef CS
-#endif
-
 namespace llvm {
 class APInt;
 template <typename T> class ArrayRef;
