@@ -14,14 +14,23 @@
 #ifndef LLVM_CONFIG_H
 #define LLVM_CONFIG_H
 
+/*
+ * When bsd.lib.mk builds shared libraries it builds with -DPIC which 
+ * causes problems in the following files which use PIC as a variable name.
+ * undefine PIC here to minimise the diff to upstream LLVM
+ *
+ * include/llvm/MC/MCObjectFileInfo.h
+ * lib/MC/MCObjectFileInfo.cpp
+ * lib/Transforms/Scalar/LICM.cpp
+ * lib/Transforms/Utils/PredicateInfo.cpp
+ */
+#undef PIC
+
 /* Define if LLVM_ENABLE_DUMP is enabled */
 /* #undef LLVM_ENABLE_DUMP */
 
-/* Define if we link Polly to the tools */
-/* #undef LINK_POLLY_INTO_TOOLS */
-
 /* Target triple LLVM will generate code for by default */
-#define LLVM_DEFAULT_TARGET_TRIPLE "armv6-unknown-netbsdelf9.99-eabihf"
+#define LLVM_DEFAULT_TARGET_TRIPLE "sparc64-unknown-openbsd6.8"
 
 /* Define if threads enabled */
 #define LLVM_ENABLE_THREADS 1
@@ -30,28 +39,28 @@
 #define LLVM_HAS_ATOMICS 1
 
 /* Host triple LLVM will be executed on */
-#define LLVM_HOST_TRIPLE "armv6-unknown-netbsdelf9.99-eabihf"
+#define LLVM_HOST_TRIPLE "sparc64-unknown-openbsd6.8"
 
 /* LLVM architecture name for the native architecture, if available */
-#define LLVM_NATIVE_ARCH ARM
+#define LLVM_NATIVE_ARCH Sparc
 
 /* LLVM name for the native AsmParser init function, if available */
-#define LLVM_NATIVE_ASMPARSER LLVMInitializeARMAsmParser
+#define LLVM_NATIVE_ASMPARSER LLVMInitializeSparcAsmParser
 
 /* LLVM name for the native AsmPrinter init function, if available */
-#define LLVM_NATIVE_ASMPRINTER LLVMInitializeARMAsmPrinter
+#define LLVM_NATIVE_ASMPRINTER LLVMInitializeSparcAsmPrinter
 
 /* LLVM name for the native Disassembler init function, if available */
-#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeARMDisassembler
+#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeSparcDisassembler
 
 /* LLVM name for the native Target init function, if available */
-#define LLVM_NATIVE_TARGET LLVMInitializeARMTarget
+#define LLVM_NATIVE_TARGET LLVMInitializeSparcTarget
 
 /* LLVM name for the native TargetInfo init function, if available */
-#define LLVM_NATIVE_TARGETINFO LLVMInitializeARMTargetInfo
+#define LLVM_NATIVE_TARGETINFO LLVMInitializeSparcTargetInfo
 
 /* LLVM name for the native target MC init function, if available */
-#define LLVM_NATIVE_TARGETMC LLVMInitializeARMTargetMC
+#define LLVM_NATIVE_TARGETMC LLVMInitializeSparcTargetMC
 
 /* Define if this is Unixish platform */
 #define LLVM_ON_UNIX 1
@@ -66,16 +75,16 @@
 #define LLVM_USE_PERF 0
 
 /* Major version of the LLVM API */
-#define LLVM_VERSION_MAJOR 9
+#define LLVM_VERSION_MAJOR 10
 
 /* Minor version of the LLVM API */
 #define LLVM_VERSION_MINOR 0
 
 /* Patch version of the LLVM API */
-#define LLVM_VERSION_PATCH 0
+#define LLVM_VERSION_PATCH 1
 
 /* LLVM version string */
-#define LLVM_VERSION_STRING "9.0.0"
+#define LLVM_VERSION_STRING "10.0.1"
 
 /* Whether LLVM records statistics for use with GetStatistics(),
  * PrintStatistics() or PrintStatisticsJSON()
